# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0004_initial'),
        ('coverageeligibilityresponses', '0001_initial'),
        ('coverages', '0002_initial'),
        ('patients', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='coverageeligibilityresponse',
            name='patient',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='coverage_eligibility_response_patient', to='patients.patientreference'),
        ),
        migrations.AddField(
            model_name='coverageeligibilityresponse',
            name='serviced_period',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='coverage_eligibility_response_serviced_period', to='base.period'),
        ),
        migrations.AddField(
            model_name='coverageeligibilityresponseerror',
            name='code',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='coverage_eligibility_response_error_code', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='coverageeligibilityresponse',
            name='error',
            field=models.ManyToManyField(blank=True, related_name='coverage_eligibility_response_error', to='coverageeligibilityresponses.coverageeligibilityresponseerror'),
        ),
        migrations.AddField(
            model_name='coverageeligibilityresponseevent',
            name='type',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='coverage_eligibility_response_event_type', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='coverageeligibilityresponseevent',
            name='when_period',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='coverage_eligibility_response_event_when_period', to='base.period'),
        ),
        migrations.AddField(
            model_name='coverageeligibilityresponse',
            name='event',
            field=models.ManyToManyField(blank=True, related_name='coverage_eligibility_response_event', to='coverageeligibilityresponses.coverageeligibilityresponseevent'),
        ),
        migrations.AddField(
            model_name='coverageeligibilityresponseinsurance',
            name='benefit_period',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='coverage_eligibility_response_insurance_benefit_period', to='base.period'),
        ),
        migrations.AddField(
            model_name='coverageeligibilityresponseinsurance',
            name='coverage',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='coverage_eligibility_response_insurance_coverage', to='coverages.coveragereference'),
        ),
        migrations.AddField(
            model_name='coverageeligibilityresponse',
            name='insurance',
            field=models.ManyToManyField(blank=True, related_name='coverage_eligibility_response_insurance', to='coverageeligibilityresponses.coverageeligibilityresponseinsurance'),
        ),
        migrations.AddField(
            model_name='coverageeligibilityresponseinsuranceitem',
            name='authorization_supporting',
            field=models.ManyToManyField(blank=True, related_name='coverage_eligibility_response_insurance_item_authorization_supporting', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='coverageeligibilityresponseinsuranceitem',
            name='category',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='coverage_eligibility_response_insurance_item_category', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='coverageeligibilityresponseinsuranceitem',
            name='modifier',
            field=models.ManyToManyField(related_name='coverage_eligibility_response_insurance_item_modifier', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='coverageeligibilityresponseinsuranceitem',
            name='network',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='coverage_eligibility_response_insurance_item_network', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='coverageeligibilityresponseinsuranceitem',
            name='product_or_service',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='coverage_eligibility_response_insurance_item_product_or_service', to='base.codeableconcept'),
        ),
    ]



"""Enrollment responses serializers."""

from dfhir.base.serializers import (
    BaseReferenceModelSerializer,
    BaseWritableNestedModelSerializer,
    IdentifierSerializer,
    OrganizationReferenceSerializer,
)
from dfhir.enrollmentrequests.serializers import EnrollmentRequestReferenceSerializer

from .models import (
    EnrollmentResponse,
    EnrollmentResponseRequestProviderReference,
)


class EnrollmentResponseRequestProviderReferenceSerializer(
    BaseReferenceModelSerializer
):
    """EnrollmentResponse Request Provider Reference Serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """Meta class."""

        model = EnrollmentResponseRequestProviderReference
        exclude = ["created_at", "updated_at"]


class EnrollmentResponseSerializer(BaseWritableNestedModelSerializer):
    """EnrollmentResponse Serializer."""

    identifier = IdentifierSerializer(many=True, required=False)
    request = EnrollmentRequestReferenceSerializer(required=False)
    organization = OrganizationReferenceSerializer(required=False)
    request_provider = EnrollmentResponseRequestProviderReferenceSerializer(
        required=False
    )

    class Meta:
        """Meta class."""

        model = EnrollmentResponse
        exclude = ["created_at", "updated_at"]



"""Healthcare Services URL Configuration."""

from django.urls import path

from . import views

app_name = "healthcareservices"

urlpatterns = [
    path("healthcareservices/", views.HealthcareServiceListView.as_view()),
    path("healthcareservices/<int:pk>/", views.HealthcareServiceDetailView.as_view()),
    path("services/category/", views.ServiceCategoryListView.as_view()),
    path(
        "services/specialty/",
        views.ClinicalSpecialtyValuesetListView.as_view(),
    ),
    path(
        "services/type/",
        views.ServiceTypeListView.as_view(),
    ),
]



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0004_initial'),
        ('careteams', '0003_initial'),
        ('coverages', '0002_initial'),
        ('devices', '0001_initial'),
        ('encounters', '0003_initial'),
        ('groups', '0001_initial'),
        ('healthcareservices', '0001_initial'),
        ('locations', '0001_initial'),
        ('organizations', '0001_initial'),
        ('patients', '0001_initial'),
        ('practitionerroles', '0001_initial'),
        ('practitioners', '0001_initial'),
        ('provenances', '0001_initial'),
        ('relatedpersons', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='TaskInput',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('value', models.CharField(max_length=255, null=True)),
                ('type', models.ForeignKey(on_delete=django.db.models.deletion.DO_NOTHING, related_name='task_input_type', to='base.codeableconcept')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='TaskOutput',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('value', models.CharField(max_length=255, null=True)),
                ('type', models.ForeignKey(on_delete=django.db.models.deletion.DO_NOTHING, related_name='task_output_type', to='base.codeableconcept')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='TaskOwnerReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
                ('care_team', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='task_owner_practitioner_care_team', to='careteams.careteam')),
                ('identifier', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='task_owner_reference_identifier', to='base.identifier')),
                ('organization', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='task_owner_reference_organization', to='organizations.organization')),
                ('patient', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='task_owner_reference_patient', to='patients.patient')),
                ('practitioner', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='task_owner_reference_practitioner', to='practitioners.practitioner')),
                ('practitioner_role', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='task_owner_reference_practitioner_role', to='practitionerroles.practitionerrole')),
                ('related_person', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='task_owner_reference_related_person', to='relatedpersons.relatedperson')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='TaskPerformer',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('function', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='task_performer_model', to='base.codeableconcept')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Task',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('instantiate_uri', models.URLField(null=True)),
                ('status', models.CharField(choices=[('draft', 'Draft'), ('requested', 'Requested'), ('received', 'Received'), ('accepted', 'Accepted'), ('rejected', 'Rejected'), ('ready', 'Ready'), ('cancelled', 'Cancelled'), ('in-progress', 'In Progress'), ('on-hold', 'On Hold'), ('failed', 'Failed'), ('completed', 'Completed'), ('entered-in-error', 'Entered in Error')], max_length=255, null=True)),
                ('intent', models.CharField(choices=[('unknown', 'unknown'), ('proposal', 'proposal'), ('plan', 'plan'), ('order', 'order'), ('original-order', 'original-order'), ('reflex-order', 'reflex-order'), ('filler-order', 'filler-order'), ('instance-order', 'instance-order'), ('option', 'option')], max_length=255, null=True)),
                ('priority', models.CharField(choices=[('routine', 'routine'), ('urgent', 'urgent'), ('asap', 'asap'), ('stat', 'stat')], max_length=255, null=True)),
                ('do_not_perform', models.BooleanField(default=False)),
                ('description', models.TextField(null=True)),
                ('authored_on', models.DateTimeField(null=True)),
                ('last_modified', models.DateTimeField(null=True)),
                ('based_on', models.ManyToManyField(blank=True, related_name='task_based_on', to='base.reference')),
                ('business_status', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='task_business_status', to='base.codeableconcept')),
                ('code', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='task_code', to='base.codeableconcept')),
                ('encounter', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='task_encounter', to='encounters.encounterreference')),
                ('execution_period', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='task_execution_period', to='base.period')),
                ('focus', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='task_focus', to='base.reference')),
                ('for_value', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='task_for', to='base.reference')),
                ('group_identifier', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='task_group_identifier', to='base.identifier')),
                ('identifier', models.ManyToManyField(related_name='task_identifier', to='base.identifier')),
                ('insurance', models.ManyToManyField(blank=True, related_name='task_insurance', to='coverages.coverageclaimresponsereference')),
                ('location', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='task_location', to='locations.locationreference')),
                ('note', models.ManyToManyField(blank=True, related_name='task_note', to='base.annotation')),
                ('reason', models.ManyToManyField(blank=True, related_name='task_reason', to='base.codeablereference')),
                ('relevant_history', models.ManyToManyField(blank=True, related_name='task_relevant_history', to='provenances.provenancereference')),
                ('requested_period', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='task_requested_period', to='base.period')),
                ('status_reason', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='task_status_reason', to='base.codeablereference')),
                ('input', models.ManyToManyField(blank=True, related_name='task_input', to='tasks.taskinput')),
                ('output', models.ManyToManyField(blank=True, related_name='task_output', to='tasks.taskoutput')),
                ('owner', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='task_owner', to='tasks.taskownerreference')),
                ('performer', models.ManyToManyField(blank=True, related_name='task_performer', to='tasks.taskperformer')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='TaskPerformerActorReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
                ('care_team', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='task_performer_actor_reference_practitioner_care_team', to='careteams.careteam')),
                ('device', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='task_performer_actor_reference_device', to='devices.device')),
                ('identifier', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='task_performer_actor_reference_identifier', to='base.identifier')),
                ('organization', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='task_performer_actor_reference_organization', to='organizations.organization')),
                ('patient', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='task_performer_actor_reference_patient', to='patients.patient')),
                ('practitioner', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='task_performer_actor_reference_practitioner', to='practitioners.practitioner')),
                ('practitioner_role', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='task_performer_actor_reference_practitioner_role', to='practitionerroles.practitionerrole')),
                ('related_person', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='task_performer_actor_reference_related_person', to='relatedpersons.relatedperson')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='taskperformer',
            name='actor',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='task_performer_actor', to='tasks.taskperformeractorreference'),
        ),
        migrations.CreateModel(
            name='TaskReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
                ('identifier', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='task_reference_identifier', to='base.identifier')),
                ('task', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='task_reference_task', to='tasks.task')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='task',
            name='part_of',
            field=models.ManyToManyField(blank=True, related_name='task_part_of', to='tasks.taskreference'),
        ),
        migrations.CreateModel(
            name='TaskRequestedPerformerCodeableReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('concept', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='task_requested_performer_codeable_reference_concept', to='base.codeableconcept')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='task',
            name='requested_performer',
            field=models.ManyToManyField(blank=True, related_name='task_requested_performer', to='tasks.taskrequestedperformercodeablereference'),
        ),
        migrations.CreateModel(
            name='TaskRequestedPerformerReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
                ('care_team', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='task_requested_practitioner_care_team', to='careteams.careteam')),
                ('device', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='task_requested_performer_reference_device', to='devices.device')),
                ('healthcare_service', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='task_requested_practitioner_healthcare_service', to='healthcareservices.healthcareservice')),
                ('identifier', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='task_requested_performer_reference_identifier', to='base.identifier')),
                ('organization', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='task_requested_performer_reference_organization', to='organizations.organization')),
                ('patient', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='task_requested_performer_reference_patient', to='patients.patient')),
                ('practitioner', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='task_requested_performer_reference_practitioner', to='practitioners.practitioner')),
                ('practitioner_role', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='task_requested_performer_reference_practitioner_role', to='practitionerroles.practitionerrole')),
                ('related_person', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='task_requested_performer_reference_related_person', to='relatedpersons.relatedperson')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='taskrequestedperformercodeablereference',
            name='reference',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='task_requested_performer_codeable_reference_reference', to='tasks.taskrequestedperformerreference'),
        ),
        migrations.CreateModel(
            name='TaskRequesterReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
                ('device', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='task_requester_reference_device', to='devices.device')),
                ('identifier', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='task_requester_reference_identifier', to='base.identifier')),
                ('organization', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='task_requester_reference_organization', to='organizations.organization')),
                ('patient', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='task_requester_reference_patient', to='patients.patient')),
                ('practitioner', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='task_requester_reference_practitioner', to='practitioners.practitioner')),
                ('practitioner_role', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='task_requester_reference_practitioner_role', to='practitionerroles.practitionerrole')),
                ('related_person', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='task_requester_reference_related_person', to='relatedpersons.relatedperson')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='task',
            name='requester',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='task_requester', to='tasks.taskrequesterreference'),
        ),
        migrations.CreateModel(
            name='TaskRestriction',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('repetition', models.PositiveIntegerField(null=True)),
                ('period', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='task_restriction_period', to='base.period')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='task',
            name='restriction',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='task_restriction', to='tasks.taskrestriction'),
        ),
        migrations.CreateModel(
            name='TaskRestrictionRecipientReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
                ('group', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='task_requested_practitioner_care_team', to='groups.group')),
                ('identifier', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='task_restriction_recipient_reference', to='base.identifier')),
                ('organization', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='task_restriction_recipient_reference_organization', to='organizations.organization')),
                ('patient', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='task_restriction_recipient_reference_patient', to='patients.patient')),
                ('practitioner', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='task_restriction_recipient_reference_practitioner', to='practitioners.practitioner')),
                ('practitioner_role', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='task_restriction_recipient_reference_practitioner_role', to='practitionerroles.practitionerrole')),
                ('related_person', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='task_restriction_recipient_reference_related_person', to='relatedpersons.relatedperson')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='taskrestriction',
            name='recipient',
            field=models.ManyToManyField(blank=True, related_name='ask_restrictions_recipient', to='tasks.taskrestrictionrecipientreference'),
        ),
    ]



"""imaging study models."""

from django.db import models

from dfhir.base.models import (
    BaseReference,
    CodeableConcept,
    Identifier,
    TimeStampedModel,
)
from dfhir.imagingstudies.choices import ImagingStudyStatusChoices


class ImagingStudySubjectReference(BaseReference):
    """Imaging study subject reference."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        related_name="imaging_study_subject_reference",
        null=True,
    )
    patient = models.ForeignKey(
        "patients.Patient",
        on_delete=models.DO_NOTHING,
        related_name="imaging_study_subject_reference",
        null=True,
    )
    device = models.ForeignKey(
        "devices.Device",
        on_delete=models.DO_NOTHING,
        related_name="imaging_study_subject_reference",
        null=True,
    )
    group = models.ForeignKey(
        "groups.Group",
        on_delete=models.DO_NOTHING,
        related_name="imaging_study_subject_reference",
        null=True,
    )


class ImagingStudyBasedOnReference(BaseReference):
    """Imaging study based on reference."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        related_name="imaging_study_based_on_reference",
        null=True,
    )
    care_plan = models.ForeignKey(
        "careplans.CarePlan",
        on_delete=models.DO_NOTHING,
        related_name="imaging_study_based_on_reference",
        null=True,
    )
    service_request = models.ForeignKey(
        "servicerequests.ServiceRequest",
        on_delete=models.DO_NOTHING,
        related_name="imaging_study_based_on_reference",
        null=True,
    )
    appointment = models.ForeignKey(
        "appointments.Appointment",
        on_delete=models.DO_NOTHING,
        related_name="imaging_study_based_on_reference",
        null=True,
    )
    appointment_response = models.ForeignKey(
        "appointmentresponses.AppointmentResponse",
        on_delete=models.DO_NOTHING,
        related_name="imaging_study_based_on_reference",
        null=True,
    )
    task = models.ForeignKey(
        "tasks.Task",
        on_delete=models.DO_NOTHING,
        related_name="imaging_study_based_on_reference",
        null=True,
    )


class ImagingStudyProcedureReference(BaseReference):
    """Imaging study procedure reference."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        related_name="imaging_study_procedure_reference",
        null=True,
    )

    # TODO: plan_definition = models.ForeignKey(
    #     "plandefinitions.PlanDefinition",
    #     on_delete=models.DO_NOTHING,
    #     related_name="imaging_study_procedure_reference",
    #     null=True,
    # )

    # TODO: activity_definition = models.ForeignKey(
    #     "activitydefinitions.ActivityDefinition",
    #     on_delete=models.DO_NOTHING,
    #     related_name="imaging_study_procedure_reference",
    #     null=True,
    # )


class ImagingStudyReasonReference(BaseReference):
    """imaging study reason."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        related_name="imaging_study_reason",
        null=True,
    )
    condition = models.ForeignKey(
        "conditions.Condition",
        on_delete=models.DO_NOTHING,
        related_name="imaging_study_reason",
        null=True,
    )
    observation = models.ForeignKey(
        "observations.Observation",
        on_delete=models.DO_NOTHING,
        related_name="imaging_study_reason",
        null=True,
    )
    diagnostic_report = models.ForeignKey(
        "diagnosticreports.DiagnosticReport",
        on_delete=models.DO_NOTHING,
        related_name="imaging_study_reason",
        null=True,
    )
    document_reference = models.ForeignKey(
        "documentreferences.DocumentReference",
        on_delete=models.DO_NOTHING,
        related_name="imaging_study_reason",
        null=True,
    )


class ImagingStudySeriesPerformerActorReference(BaseReference):
    """imaging study series performer reference."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        related_name="imaging_study_series_performer_reference",
        null=True,
    )
    practitioner = models.ForeignKey(
        "practitioners.Practitioner",
        on_delete=models.DO_NOTHING,
        related_name="imaging_study_series_performer_reference",
        null=True,
    )
    practitioner_role = models.ForeignKey(
        "practitionerroles.PractitionerRole",
        on_delete=models.DO_NOTHING,
        related_name="imaging_study_series_performer_reference",
        null=True,
    )
    organization = models.ForeignKey(
        "organizations.Organization",
        on_delete=models.DO_NOTHING,
        related_name="imaging_study_series_performer_reference",
        null=True,
    )
    care_team = models.ForeignKey(
        "careteams.CareTeam",
        on_delete=models.DO_NOTHING,
        related_name="imaging_study_series_performer_reference",
        null=True,
    )
    patient = models.ForeignKey(
        "patients.Patient",
        on_delete=models.DO_NOTHING,
        related_name="imaging_study_series_performer_reference",
        null=True,
    )
    device = models.ForeignKey(
        "devices.Device",
        on_delete=models.DO_NOTHING,
        related_name="imaging_study_series_performer_reference",
        null=True,
    )
    related_person = models.ForeignKey(
        "relatedpersons.RelatedPerson",
        on_delete=models.DO_NOTHING,
        related_name="imaging_study_series_performer_reference",
        null=True,
    )
    healthcare_service = models.ForeignKey(
        "healthcareservices.HealthcareService",
        on_delete=models.DO_NOTHING,
        related_name="imaging_study_series_performer_reference",
        null=True,
    )


class ImagingStudySeriesPerformer(TimeStampedModel):
    """imaging study series performer."""

    function = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        related_name="imaging_study_series_performer_reference",
        null=True,
    )
    actor = models.ForeignKey(
        ImagingStudySeriesPerformerActorReference,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="imaging_study_series_performer_reference",
    )


class ImagingStudySeriesInstance(TimeStampedModel):
    """imaging study series instance model."""

    uid = models.CharField(max_length=255, null=False)
    sop_class = models.CharField(max_length=255, null=True)
    number = models.IntegerField(null=True)
    title = models.CharField(max_length=255, null=True)


class ImagingStudySeries(TimeStampedModel):
    """Imaging study series."""

    uid = models.CharField(max_length=255, null=True)
    number = models.IntegerField(null=True)
    modality = models.ForeignKey(
        CodeableConcept,
        null=True,
        on_delete=models.DO_NOTHING,
        related_name="imaging_study_series_modality",
    )
    description = models.CharField(max_length=255, null=True)
    number_of_instances = models.IntegerField(null=True)
    endpoint = models.ManyToManyField(
        "endpoints.EndpointReference", related_name="imaging_study_series_endpoint"
    )
    body_site = models.ForeignKey(
        "bodystructures.BodyStructureCodeableReference",
        null=True,
        on_delete=models.DO_NOTHING,
        related_name="imaging_study_series_body_site",
    )
    laterality = models.ForeignKey(
        CodeableConcept,
        null=True,
        on_delete=models.DO_NOTHING,
        related_name="imaging_study_series_laterality",
    )
    specimen = models.ManyToManyField(
        "specimens.SpecimenReference",
        blank=True,
        related_name="imaging_study_series_specimen",
    )
    started = models.DateTimeField(null=True)
    performer = models.ManyToManyField(
        ImagingStudySeriesPerformer,
        related_name="imaging_study_series_performer",
        blank=True,
    )
    instance = models.ManyToManyField(
        ImagingStudySeriesInstance,
        blank=True,
        related_name="imaging_study_series_instance",
    )


class ImagingStudy(TimeStampedModel):
    """imaging study model."""

    identifier = models.ManyToManyField(
        Identifier, related_name="imaging_study_identifier", blank=True
    )
    status = models.CharField(
        max_length=255, null=True, choices=ImagingStudyStatusChoices.choices
    )
    modality = models.ManyToManyField(
        CodeableConcept, related_name="imaging_study_modality", blank=True
    )
    subject = models.ForeignKey(
        ImagingStudySubjectReference,
        null=True,
        on_delete=models.DO_NOTHING,
        related_name="imaging_study_subject",
    )
    encounter = models.ForeignKey(
        "encounters.EncounterReference",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="imaging_study_encounter",
    )
    started = models.DateTimeField(null=True)
    based_on = models.ManyToManyField(
        ImagingStudyBasedOnReference, related_name="imaging_study_based_on", blank=True
    )
    part_of = models.ManyToManyField(
        "procedures.ProcedureReference",
        related_name="imaging_study_part_of",
        blank=True,
    )
    referrer = models.ForeignKey(
        "practitioners.PractitionerPractitionerRoleReference",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="imaging_study_referrer",
    )
    endpoint = models.ManyToManyField(
        "endpoints.EndpointReference", related_name="imaging_study_endpoint", blank=True
    )
    procedure = models.ManyToManyField(
        ImagingStudyProcedureReference,
        related_name="imaging_study_procedure",
        blank=True,
    )
    location = models.ForeignKey(
        "locations.LocationReference",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="imaging_study_location",
    )
    reason = models.ManyToManyField(
        ImagingStudyReasonReference, blank=True, related_name="imaging_study_reason"
    )
    note = models.ManyToManyField(
        "base.Annotation", blank=True, related_name="imaging_study_note"
    )
    description = models.CharField(max_length=255, null=True)
    number_of_series = models.IntegerField(null=True)
    number_of_instances = models.IntegerField(null=True)
    series = models.ManyToManyField(
        ImagingStudySeries, related_name="imaging_study_series", blank=True
    )


class ImagingStudyReference(BaseReference):
    """imaging selection reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        related_name="imaging_study_reference_identifier",
        null=True,
    )
    imaging_study = models.ForeignKey(
        ImagingStudy,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="imaging_study_reference_imaging_study",
    )



"""Endpoints urls."""

from django.urls import path

from . import views

app_name = "endpoints"

urlpatterns = [
    path("endpoints/", views.EndpointListView.as_view()),
    path("endpoints/<int:pk>/", views.EndpointDetailView.as_view()),
]



"""Adverse events models."""

from django.db import models

from dfhir.base.models import (
    Annotation,
    BaseReference,
    CodeableConcept,
    Identifier,
    Period,
    TimeStampedModel,
)

from . import choices


class AdverseEventSubjectReference(BaseReference):
    """Adverse event subject reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.CASCADE,
        related_name="adverse_event_subject_reference_identifier",
        null=True,
    )
    patient = models.ForeignKey(
        "patients.Patient",
        on_delete=models.CASCADE,
        related_name="adverse_event_subject_reference_patient",
        null=True,
    )
    group = models.ForeignKey(
        "groups.Group",
        on_delete=models.CASCADE,
        related_name="adverse_event_subject_reference_group",
        null=True,
    )
    practitioner = models.ForeignKey(
        "practitioners.Practitioner",
        on_delete=models.CASCADE,
        related_name="adverse_event_subject_reference_practitioner",
        null=True,
    )
    related_person = models.ForeignKey(
        "relatedpersons.RelatedPerson",
        on_delete=models.CASCADE,
        related_name="adverse_event_subject_reference_related_person",
        null=True,
    )


class AdverseEventRecorderReference(BaseReference):
    """Adverse event recorder reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.CASCADE,
        related_name="adverse_event_recorder_reference_identifier",
        null=True,
    )
    patient = models.ForeignKey(
        "patients.Patient",
        on_delete=models.CASCADE,
        related_name="adverse_event_recorder_reference_patient",
        null=True,
    )
    practitioner = models.ForeignKey(
        "practitioners.Practitioner",
        on_delete=models.CASCADE,
        related_name="adverse_event_recorder_reference_practitioner",
        null=True,
    )
    practitioner_role = models.ForeignKey(
        "practitionerroles.PractitionerRole",
        on_delete=models.CASCADE,
        related_name="adverse_event_recorder_reference_practitioner_role",
        null=True,
    )
    related_person = models.ForeignKey(
        "relatedpersons.RelatedPerson",
        on_delete=models.CASCADE,
        related_name="adverse_event_recorder_reference_related_person",
        null=True,
    )


class AdverseEventContributingFactorReference(BaseReference):
    """Adverse event contributing factor reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.CASCADE,
        related_name="adverse_event_contributing_factor_reference_identifier",
        null=True,
    )
    condition = models.ForeignKey(
        "conditions.Condition",
        on_delete=models.CASCADE,
        related_name="adverse_event_contributing_factor_reference_condition",
        null=True,
    )
    observation = models.ForeignKey(
        "observations.Observation",
        on_delete=models.CASCADE,
        related_name="adverse_event_contributing_factor_reference_observation",
        null=True,
    )
    allergy_intolerance = models.ForeignKey(
        "allergyintolerances.AllergyIntolerance",
        on_delete=models.CASCADE,
        related_name="adverse_event_contributing_factor_reference_allergy_intolerance",
        null=True,
    )
    family_member_history = models.ForeignKey(
        "familymemberhistories.FamilyMemberHistory",
        on_delete=models.CASCADE,
        related_name="adverse_event_contributing_factor_reference_family_member_history",
        null=True,
    )
    immunization = models.ForeignKey(
        "immunizations.Immunization",
        on_delete=models.CASCADE,
        related_name="adverse_event_contributing_factor_reference_immunization",
        null=True,
    )
    procedure = models.ForeignKey(
        "procedures.Procedure",
        on_delete=models.CASCADE,
        related_name="adverse_event_contributing_factor_reference_procedure",
        null=True,
    )
    device = models.ForeignKey(
        "devices.Device",
        on_delete=models.CASCADE,
        related_name="adverse_event_contributing_factor_reference_device",
        null=True,
    )
    device_usage = models.ForeignKey(
        "deviceusages.DeviceUsage",
        on_delete=models.CASCADE,
        related_name="adverse_event_contributing_factor_reference_device_usage",
        null=True,
    )
    document_reference = models.ForeignKey(
        "documentreferences.DocumentReference",
        on_delete=models.CASCADE,
        related_name="adverse_event_contributing_factor_reference_document_reference",
        null=True,
    )
    medication_administration = models.ForeignKey(
        "medicationadministrations.MedicationAdministration",
        on_delete=models.CASCADE,
        related_name="adverse_event_contributing_factor_reference_medication_administration",
        null=True,
    )
    medication_statement = models.ForeignKey(
        "medicationstatements.MedicationStatement",
        on_delete=models.CASCADE,
        related_name="adverse_event_contributing_factor_reference_medication_statement",
        null=True,
    )


class AdverseEventContributingFactorCodeableReference(TimeStampedModel):
    """Adverse event contributing factor codeable reference model."""

    reference = models.ForeignKey(
        AdverseEventContributingFactorReference,
        on_delete=models.CASCADE,
        related_name="adverse_event_contributing_factor_codeable_reference_reference",
        null=True,
    )
    concept = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="adverse_event_contributing_factor_codeable_reference_concept",
        null=True,
    )


class AdverseEventPreventiveActionReference(BaseReference):
    """Adverse event preventive action reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.CASCADE,
        related_name="adverse_event_preventive_action_reference_identifier",
        null=True,
    )
    immunization = models.ForeignKey(
        "immunizations.Immunization",
        on_delete=models.CASCADE,
        related_name="adverse_event_preventive_action_reference_immunization",
        null=True,
    )
    procedure = models.ForeignKey(
        "procedures.Procedure",
        on_delete=models.CASCADE,
        related_name="adverse_event_preventive_action_reference_procedure",
        null=True,
    )
    document_reference = models.ForeignKey(
        "documentreferences.DocumentReference",
        on_delete=models.CASCADE,
        related_name="adverse_event_preventive_action_reference_document_reference",
        null=True,
    )
    medication_administration = models.ForeignKey(
        "medicationadministrations.MedicationAdministration",
        on_delete=models.CASCADE,
        related_name="adverse_event_preventive_action_reference_medication_administration",
        null=True,
    )
    medication_request = models.ForeignKey(
        "medicationrequests.MedicationRequest",
        on_delete=models.CASCADE,
        related_name="adverse_event_preventive_action_reference_medication_request",
        null=True,
    )


class AdverseEventPreventiveActionCodeableReference(TimeStampedModel):
    """Adverse event preventive action codeable reference model."""

    reference = models.ForeignKey(
        AdverseEventPreventiveActionReference,
        on_delete=models.CASCADE,
        related_name="adverse_event_preventive_action_codeable_reference_reference",
        null=True,
    )
    concept = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="adverse_event_preventive_action_codeable_reference_concept",
        null=True,
    )


class AdverseEventMitigatingActionReference(BaseReference):
    """Adverse event mitigating action reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.CASCADE,
        related_name="adverse_event_mitigating_action_reference_identifier",
        null=True,
    )
    procedure = models.ForeignKey(
        "procedures.Procedure",
        on_delete=models.CASCADE,
        related_name="adverse_event_mitigating_action_reference_procedure",
        null=True,
    )
    document_reference = models.ForeignKey(
        "documentreferences.DocumentReference",
        on_delete=models.CASCADE,
        related_name="adverse_event_mitigating_action_reference_document_reference",
        null=True,
    )
    medication_administration = models.ForeignKey(
        "medicationadministrations.MedicationAdministration",
        on_delete=models.CASCADE,
        related_name="adverse_event_mitigating_action_reference_medication_administration",
        null=True,
    )
    medication_request = models.ForeignKey(
        "medicationrequests.MedicationRequest",
        on_delete=models.CASCADE,
        related_name="adverse_event_mitigating_action_reference_medication_request",
        null=True,
    )


class AdverseEventMitigatingActionCodeableReference(TimeStampedModel):
    """Adverse event mitigating action codeable reference model."""

    reference = models.ForeignKey(
        AdverseEventMitigatingActionReference,
        on_delete=models.CASCADE,
        related_name="adverse_event_mitigating_action_codeable_reference_reference",
        null=True,
    )
    concept = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="adverse_event_mitigating_action_codeable_reference_concept",
        null=True,
    )


class AdverseEventSupportingInfoReference(BaseReference):
    """Adverse event supporting info reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.CASCADE,
        related_name="adverse_event_supporting_info_reference_identifier",
        null=True,
    )
    condition = models.ForeignKey(
        "conditions.Condition",
        on_delete=models.CASCADE,
        related_name="adverse_event_supporting_info_reference_condition",
        null=True,
    )
    observation = models.ForeignKey(
        "observations.Observation",
        on_delete=models.CASCADE,
        related_name="adverse_event_supporting_info_reference_observation",
        null=True,
    )
    allergy_intolerance = models.ForeignKey(
        "allergyintolerances.AllergyIntolerance",
        on_delete=models.CASCADE,
        related_name="adverse_event_supporting_info_reference_allergy_intolerance",
        null=True,
    )
    family_member_history = models.ForeignKey(
        "familymemberhistories.FamilyMemberHistory",
        on_delete=models.CASCADE,
        related_name="adverse_event_supporting_info_reference_family_member_history",
        null=True,
    )
    immunization = models.ForeignKey(
        "immunizations.Immunization",
        on_delete=models.CASCADE,
        related_name="adverse_event_supporting_info_reference_immunization",
        null=True,
    )
    procedure = models.ForeignKey(
        "procedures.Procedure",
        on_delete=models.CASCADE,
        related_name="adverse_event_supporting_info_reference_procedure",
        null=True,
    )
    document_reference = models.ForeignKey(
        "documentreferences.DocumentReference",
        on_delete=models.CASCADE,
        related_name="adverse_event_supporting_info_reference_document_reference",
        null=True,
    )
    medication_administration = models.ForeignKey(
        "medicationadministrations.MedicationAdministration",
        on_delete=models.CASCADE,
        related_name="adverse_event_supporting_info_reference_medication_administration",
        null=True,
    )
    medication_statement = models.ForeignKey(
        "medicationstatements.MedicationStatement",
        on_delete=models.CASCADE,
        related_name="adverse_event_supporting_info_reference_medication_statement",
        null=True,
    )
    # questionnaire_response = models.ForeignKey(
    #     "questionnaireresponses.QuestionnaireResponse",
    #     on_delete=models.CASCADE,
    #     related_name="adverse_event_supporting_info_reference_questionnaire_response",
    #     null=True,
    # )


class AdverseEventSupportingInfoCodeableReference(TimeStampedModel):
    """Adverse Event supporting info codeable reference."""

    reference = models.ForeignKey(
        "AdverseEventSupportingInfoReference",
        on_delete=models.CASCADE,
        related_name="adverse_event_supporting_info_codeable_reference_reference",
        null=True,
    )
    concept = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="adverse_event_supporting_info_codeable_reference_concept",
        null=True,
    )


class AdverseEventSuspectEntityInstanceReference(BaseReference):
    """Adverse event suspect entity instance reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.CASCADE,
        related_name="adverse_event_suspect_entity_instance_reference_identifier",
        null=True,
    )
    immunization = models.ForeignKey(
        "immunizations.Immunization",
        on_delete=models.CASCADE,
        related_name="adverse_event_suspect_entity_instance_reference_immunization",
        null=True,
    )
    procedure = models.ForeignKey(
        "procedures.Procedure",
        on_delete=models.CASCADE,
        related_name="adverse_event_suspect_entity_instance_reference_procedure",
        null=True,
    )
    substance = models.ForeignKey(
        "substances.Substance",
        on_delete=models.CASCADE,
        related_name="adverse_event_suspect_entity_instance_reference_substance",
        null=True,
    )
    medication = models.ForeignKey(
        "medications.Medication",
        on_delete=models.CASCADE,
        related_name="adverse_event_suspect_entity_instance_reference_medication",
        null=True,
    )
    medication_administration = models.ForeignKey(
        "medicationadministrations.MedicationAdministration",
        on_delete=models.CASCADE,
        related_name="adverse_event_suspect_entity_instance_reference_medication_administration",
        null=True,
    )
    medication_statement = models.ForeignKey(
        "medicationstatements.MedicationStatement",
        on_delete=models.CASCADE,
        related_name="adverse_event_suspect_entity_instance_reference_medication_statement",
        null=True,
    )
    device = models.ForeignKey(
        "devices.Device",
        on_delete=models.CASCADE,
        related_name="adverse_event_suspect_entity_instance_reference_device",
        null=True,
    )
    biollogically_derived_product = models.ForeignKey(
        "biologicallyderivedproducts.BiologicallyDerivedProduct",
        on_delete=models.CASCADE,
        related_name="adverse_event_suspect_entity_instance_reference_biologically_derived_product",
        null=True,
    )
    # research_study = models.ForeignKey(
    #     "researchstudies.ResearchStudy",
    #     on_delete=models.CASCADE,
    #     related_name="adverse_event_suspect_entity_instance_reference_research_study",
    #     null=True,
    # )


class AdverseEventSuspectEntityInstanceCodeableReference(TimeStampedModel):
    """Adverse event suspect entity instance codeable reference model."""

    reference = models.ForeignKey(
        AdverseEventSuspectEntityInstanceReference,
        on_delete=models.CASCADE,
        related_name="adverse_event_suspect_entity_instance_codeable_reference_reference",
        null=True,
    )
    concept = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="adverse_event_suspect_entity_instance_codeable_reference_concept",
        null=True,
    )


class AdverseEventSuspectEntityAuthorReference(BaseReference):
    """Adverse event suspect entity author reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.CASCADE,
        related_name="adverse_event_suspect_entity_author_reference_identifier",
        null=True,
    )
    practitioner = models.ForeignKey(
        "practitioners.Practitioner",
        on_delete=models.CASCADE,
        related_name="adverse_event_suspect_entity_author_reference_practitioner",
        null=True,
    )
    practitioner_role = models.ForeignKey(
        "practitionerroles.PractitionerRole",
        on_delete=models.CASCADE,
        related_name="adverse_event_suspect_entity_author_reference_practitioner_role",
        null=True,
    )
    patient = models.ForeignKey(
        "patients.Patient",
        on_delete=models.CASCADE,
        related_name="adverse_event_suspect_entity_author_reference_patient",
        null=True,
    )
    related_person = models.ForeignKey(
        "relatedpersons.RelatedPerson",
        on_delete=models.CASCADE,
        related_name="adverse_event_suspect_entity_author_reference_related_person",
        null=True,
    )


class AdverseEventSuspectEntityCausality(TimeStampedModel):
    """Adverse event suspect entity causality model."""

    assessment_method = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="adverse_event_suspect_entity_causality_assessment",
        null=True,
    )
    entity_relatedness = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="adverse_event_suspect_entity_causality_entity_relatedness",
        null=True,
    )
    author = models.ForeignKey(
        AdverseEventSuspectEntityAuthorReference,
        on_delete=models.CASCADE,
        related_name="adverse_event_suspect_entity_causality_author",
        null=True,
    )


class AdverseEventSuspectEntity(TimeStampedModel):
    """Adverse event suspect entity model."""

    instance = models.ForeignKey(
        AdverseEventSuspectEntityInstanceCodeableReference,
        on_delete=models.DO_NOTHING,
        related_name="adverse_event_suspect_entity_instance",
        null=True,
    )
    causality = models.ForeignKey(
        AdverseEventSuspectEntityCausality,
        on_delete=models.DO_NOTHING,
        related_name="adverse_event_suspect_entity_causality",
        null=True,
    )


class AdverseEventParticipantActorReference(BaseReference):
    """Adverse event participant actor reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.CASCADE,
        related_name="adverse_event_participant_actor_reference_identifier",
        null=True,
    )
    practitioner = models.ForeignKey(
        "practitioners.Practitioner",
        on_delete=models.CASCADE,
        related_name="adverse_event_participant_actor_reference_practitioner",
        null=True,
    )
    practitioner_role = models.ForeignKey(
        "practitionerroles.PractitionerRole",
        on_delete=models.CASCADE,
        related_name="adverse_event_participant_actor_reference_practitioner_role",
        null=True,
    )
    organization = models.ForeignKey(
        "organizations.Organization",
        on_delete=models.CASCADE,
        related_name="adverse_event_participant_actor_reference_organization",
        null=True,
    )
    care_team = models.ForeignKey(
        "careteams.CareTeam",
        on_delete=models.CASCADE,
        related_name="adverse_event_participant_actor_reference_care_team",
        null=True,
    )
    patient = models.ForeignKey(
        "patients.Patient",
        on_delete=models.CASCADE,
        related_name="adverse_event_participant_actor_reference_patient",
        null=True,
    )
    device = models.ForeignKey(
        "devices.Device",
        on_delete=models.CASCADE,
        related_name="adverse_event_participant_actor_reference_device",
        null=True,
    )
    related_person = models.ForeignKey(
        "relatedpersons.RelatedPerson",
        on_delete=models.CASCADE,
        related_name="adverse_event_participant_actor_reference_related_person",
        null=True,
    )


class AdverseEventParticipant(TimeStampedModel):
    """Adverse event participant model."""

    function = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        related_name="adverse_event_participant_function",
        null=True,
    )
    actor = models.ForeignKey(
        AdverseEventParticipantActorReference,
        on_delete=models.DO_NOTHING,
        related_name="adverse_event_participant_actor",
        null=True,
    )


class AdverseEvent(TimeStampedModel):
    """Adverse event model."""

    identifier = models.ManyToManyField(
        Identifier,
        related_name="adverse_event_identifier",
        blank=True,
    )
    status = models.CharField(
        max_length=255, choices=choices.AdverseEventStatusChoices.choices
    )
    actuality = models.CharField(
        max_length=255, choices=choices.AdverseEventActualityChoices.choices
    )
    category = models.ManyToManyField(
        CodeableConcept, related_name="adverse_event_category", blank=True
    )
    code = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        related_name="adverse_event_code",
        null=True,
    )
    subject = models.ForeignKey(
        AdverseEventSubjectReference,
        on_delete=models.DO_NOTHING,
        related_name="adverse_event_subject",
        null=True,
    )
    encounter = models.ForeignKey(
        "encounters.EncounterReference",
        on_delete=models.DO_NOTHING,
        related_name="adverse_event_encounter",
        null=True,
    )
    cause_date_time = models.DateTimeField(null=True)
    cause_period = models.ForeignKey(
        Period,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="adverse_event_cause_period",
    )
    effect_date_time = models.DateTimeField(null=True)
    effect_period = models.ForeignKey(
        Period,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="adverse_event_effect_period",
    )
    detected = models.DateTimeField(null=True)
    recorded_date = models.DateTimeField(null=True)
    resulting_effect = models.ManyToManyField(
        "conditions.ConditionObservationCodeableReference",
        related_name="adverse_event_resulting_effect",
        blank=True,
    )
    location = models.ForeignKey(
        "locations.LocationReference",
        on_delete=models.DO_NOTHING,
        related_name="adverse_event_location",
        null=True,
    )
    seriousness = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        related_name="adverse_event_seriousness",
        null=True,
    )
    outcome = models.ManyToManyField(
        CodeableConcept, related_name="adverse_event_outcome", blank=True
    )
    recorder = models.ForeignKey(
        AdverseEventRecorderReference,
        on_delete=models.DO_NOTHING,
        related_name="adverse_event_recorder",
        null=True,
    )
    participant = models.ManyToManyField(
        AdverseEventParticipant,
        related_name="adverse_event_participant",
        blank=True,
    )
    # study = models.ManyToManyField(
    #     "researchstudies.ResearchStudy",
    #     related_name="adverse_event_study",
    #     blank=True,
    # )
    expected_in_research_study = models.BooleanField(default=False)
    suspect_entity = models.ManyToManyField(
        AdverseEventSuspectEntity,
        related_name="adverse_event_suspect_entity",
        blank=True,
    )
    contributing_factor = models.ManyToManyField(
        AdverseEventContributingFactorCodeableReference,
        related_name="adverse_event_contributing_factor",
        blank=True,
    )
    preventive_action = models.ManyToManyField(
        AdverseEventPreventiveActionCodeableReference,
        related_name="adverse_event_preventive_action",
        blank=True,
    )
    mitigating_action = models.ManyToManyField(
        AdverseEventMitigatingActionCodeableReference,
        related_name="adverse_event_mitigating_action",
        blank=True,
    )
    supporting_info = models.ManyToManyField(
        AdverseEventSupportingInfoCodeableReference,
        related_name="adverse_event_supporting_info",
        blank=True,
    )
    note = models.ManyToManyField(
        Annotation, related_name="adverse_event_note", blank=True
    )



"""Biologically Derived Products URL Configuration."""

from django.urls import path

from . import views

app_name = "biologicallyderivedproducts"

urlpatterns = [
    path(
        "biologicallyderivedproducts/",
        views.BiologicallyDerivedProductListView.as_view(),
    ),
    path(
        "biologicallyderivedproducts/<int:pk>/",
        views.BiologicallyDerivedProductDetailView.as_view(),
    ),
]



"""Nutrition Products Choices."""

from django.db.models import TextChoices
from django.utils.translation import gettext_lazy as _


class NutritionProductStatusChoices(TextChoices):
    """Nutrition Product Status Choices."""

    ACTIVE = "active", _("Active")
    INACTIVE = "inactive", _("Inactive")
    ENTERED_IN_ERROR = "entered-in-error", _("Entered in Error")



"""nutrition intake serializers."""

from drf_writable_nested.serializers import WritableNestedModelSerializer

from dfhir.base.serializers import (
    AnnotationSerializer,
    BaseReferenceModelSerializer,
    BaseWritableNestedModelSerializer,
    CodeableConceptSerializer,
    IdentifierSerializer,
    PeriodSerializer,
    QuantitySerializer,
    RatioSerializer,
    ReferenceSerializer,
    TimingSerializer,
)
from dfhir.encounters.serializers import EncounterReferenceSerializer
from dfhir.locations.serializers import LocationReferenceSerializer
from dfhir.nutritionintakes.models import (
    NutritionIntake,
    NutritionIntakeBasedOnReference,
    NutritionIntakeCodeableReference,
    NutritionIntakeNutritionItem,
    NutritionIntakeNutritionItemConsumedItem,
    NutritionIntakeNutritionItemTotalIntake,
    NutritionIntakePartOfReference,
    NutritionIntakePerformer,
    NutritionIntakePerformerActorReference,
    NutritionIntakeReasonCodeableReference,
    NutritionIntakeReasonReference,
    NutritionIntakeReference,
    NutritionIntakeReportedReference,
)
from dfhir.nutritionproducts.serializers import (
    NutritionProductCodeableReferenceSerializer,
)
from dfhir.patients.serializers import PatientGroupReferenceSerializer


class NutritionIntakeBasedOnReferenceSerializer(BaseReferenceModelSerializer):
    """nutrition intake based on reference serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """Meta class."""

        model = NutritionIntakeBasedOnReference
        exclude = ["created_at", "updated_at"]


class NutritionIntakePartOfReferenceSerializer(BaseReferenceModelSerializer):
    """nutrition intake part of reference serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """Meta class."""

        model = NutritionIntakePartOfReference
        exclude = ["created_at", "updated_at"]


class NutritionIntakePerformerActorReferenceSerializer(BaseReferenceModelSerializer):
    """nutrition intake performer actor reference serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """meta options."""

        model = NutritionIntakePerformerActorReference
        exclude = ["created_at", "updated_at"]


class NutritionIntakeReportedReferenceSerializer(BaseReferenceModelSerializer):
    """nutrition intake reported reference serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """Meta class."""

        model = NutritionIntakeReportedReference
        exclude = ["created_at", "updated_at"]


class NutritionIntakeNutritionItemConsumedItemSerializer(WritableNestedModelSerializer):
    """nutrition intake nutrition item consumed item serializer."""

    schedule = TimingSerializer(required=False)
    amount = QuantitySerializer(required=False)
    rate_quantity = QuantitySerializer(required=False)
    rate_ratio = RatioSerializer(required=False)

    class Meta:
        """Meta class."""

        model = NutritionIntakeNutritionItemConsumedItem
        exclude = ["created_at", "updated_at"]


class NutritionIntakeNutritionItemTotalIntakeSerializer(WritableNestedModelSerializer):
    """nutrition intake nutrition item total intake serializer."""

    nutrient = CodeableConceptSerializer(required=False)
    amount = QuantitySerializer(required=False)
    energy = QuantitySerializer(required=False)

    class Meta:
        """meta option."""

        model = NutritionIntakeNutritionItemTotalIntake
        exclude = ["created_at", "updated_at"]


class NutritionIntakeNutritionItemSerializer(WritableNestedModelSerializer):
    """nutrition intake nutrition item serializer."""

    type = CodeableConceptSerializer(required=False)
    nutrition_product = NutritionProductCodeableReferenceSerializer(required=False)
    consumed_item = NutritionIntakeNutritionItemConsumedItemSerializer(
        required=False, many=True
    )
    total_intake = NutritionIntakeNutritionItemTotalIntakeSerializer(
        required=False, many=True
    )
    not_consumed_reason = CodeableConceptSerializer(required=False)

    class Meta:
        """Meta class."""

        model = NutritionIntakeNutritionItem
        exclude = ["created_at", "updated_at"]


class NutritionIntakePerformerSerializer(BaseReferenceModelSerializer):
    """nutrition intake performer reference serializer."""

    actor = NutritionIntakePerformerActorReferenceSerializer(required=False)
    function = CodeableConceptSerializer(required=False)

    class Meta:
        """Meta class."""

        model = NutritionIntakePerformer
        exclude = ["created_at", "updated_at"]


class NutritionIntakeReasonReferenceSerializer(BaseReferenceModelSerializer):
    """nutrition intake reason codeable reference serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """Meta class."""

        model = NutritionIntakeReasonReference
        exclude = ["created_at", "updated_at"]


class NutritionIntakeReasonCodeableReferenceSerializer(WritableNestedModelSerializer):
    """nutrition intake reason codeable reference serializer."""

    concept = CodeableConceptSerializer(required=False)
    reference = ReferenceSerializer(required=False)

    class Meta:
        """Meta class."""

        model = NutritionIntakeReasonCodeableReference
        exclude = ["created_at", "updated_at"]


class NutritionIntakeSerializer(BaseWritableNestedModelSerializer):
    """nutrition intake serializer."""

    identifier = IdentifierSerializer(required=False, many=True)
    based_on = NutritionIntakeBasedOnReferenceSerializer(required=False, many=True)
    part_of = NutritionIntakePartOfReferenceSerializer(required=False, many=True)
    status_reason = CodeableConceptSerializer(required=False, many=True)
    code = CodeableConceptSerializer(required=False)
    subject = PatientGroupReferenceSerializer(required=False)
    encounter = EncounterReferenceSerializer(required=False)
    occurrence_period = PeriodSerializer(required=False)
    reported_reference = NutritionIntakeReportedReferenceSerializer(required=False)
    nutrition_item = NutritionIntakeNutritionItemSerializer(required=False, many=True)
    performer = NutritionIntakePerformerSerializer(required=False, many=True)
    location = LocationReferenceSerializer(required=False)
    derived_from = ReferenceSerializer(required=False, many=True)
    reason_code = NutritionIntakeReasonCodeableReferenceSerializer(
        required=False, many=True
    )
    note = AnnotationSerializer(required=False, many=True)

    class Meta:
        """Meta class."""

        model = NutritionIntake
        exclude = ["created_at", "updated_at"]


class NutritionIntakeRefernce(BaseReferenceModelSerializer):
    """nutrition intake reference serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """Meta class."""

        model = NutritionIntakeReference
        exclude = ["created_at", "updated_at"]


class NutritionIntakeCodeableReferenceSerializer(WritableNestedModelSerializer):
    """nutrition intake codeable reference serializer."""

    concept = CodeableConceptSerializer(required=False)
    reference = ReferenceSerializer(required=False)

    class Meta:
        """Meta class."""

        model = NutritionIntakeCodeableReference
        exclude = ["created_at", "updated_at"]



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0005_initial'),
        ('conditions', '0004_initial'),
        ('diagnosticreports', '0003_initial'),
        ('immunizations', '0002_initial'),
        ('observations', '0002_initial'),
        ('organizations', '0001_initial'),
        ('patients', '0002_initial'),
        ('practitionerroles', '0002_initial'),
        ('practitioners', '0001_initial'),
        ('relatedpersons', '0001_initial'),
        ('servicerequests', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='immunizationbasedonreference',
            name='service_request',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='immunization_based_on_references', to='servicerequests.servicerequest'),
        ),
        migrations.AddField(
            model_name='immunization',
            name='based_on',
            field=models.ManyToManyField(blank=True, related_name='immunization_based_on', to='immunizations.immunizationbasedonreference'),
        ),
        migrations.AddField(
            model_name='immunizationinformationsourcerefrence',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='immunization_information_source_references', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='immunizationinformationsourcerefrence',
            name='organization',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='immunization_information_source_references', to='organizations.organization'),
        ),
        migrations.AddField(
            model_name='immunizationinformationsourcerefrence',
            name='patient',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='immunization_information_source_references', to='patients.patient'),
        ),
        migrations.AddField(
            model_name='immunizationinformationsourcerefrence',
            name='practitioner',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='immunization_information_source_references', to='practitioners.practitioner'),
        ),
        migrations.AddField(
            model_name='immunizationinformationsourcerefrence',
            name='practitioner_role',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='immunization_information_source_references', to='practitionerroles.practitionerrole'),
        ),
        migrations.AddField(
            model_name='immunizationinformationsourcerefrence',
            name='related_person',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='immunization_information_source_references', to='relatedpersons.relatedperson'),
        ),
        migrations.AddField(
            model_name='immunization',
            name='information_source',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='immunization_information_source', to='immunizations.immunizationinformationsourcerefrence'),
        ),
        migrations.AddField(
            model_name='immunizationperformer',
            name='function',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='immunization_performers', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='immunization',
            name='performer',
            field=models.ManyToManyField(blank=True, related_name='immunization_performer', to='immunizations.immunizationperformer'),
        ),
        migrations.AddField(
            model_name='immunizationperformeractorreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='immunization_performer_actor_references', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='immunizationperformeractorreference',
            name='organization',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='immunization_performer_actor_references', to='organizations.organization'),
        ),
        migrations.AddField(
            model_name='immunizationperformeractorreference',
            name='patient',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='immunization_performer_actor_references', to='patients.patient'),
        ),
        migrations.AddField(
            model_name='immunizationperformeractorreference',
            name='practitioner',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='immunization_performer_actor_references', to='practitioners.practitioner'),
        ),
        migrations.AddField(
            model_name='immunizationperformeractorreference',
            name='practitioner_role',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='immunization_performer_actor_references', to='practitionerroles.practitionerrole'),
        ),
        migrations.AddField(
            model_name='immunizationperformeractorreference',
            name='related_person',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='immunization_performer_actor_references', to='relatedpersons.relatedperson'),
        ),
        migrations.AddField(
            model_name='immunizationperformer',
            name='actor',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='immunization_performers', to='immunizations.immunizationperformeractorreference'),
        ),
        migrations.AddField(
            model_name='immunizationprogrameligibility',
            name='program',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='immunization_program_eligibility_program', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='immunizationprogrameligibility',
            name='program_status',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='immunization_program_eligibility_program_status', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='immunization',
            name='program_eligibility',
            field=models.ManyToManyField(blank=True, related_name='immunization_program_eligibility', to='immunizations.immunizationprogrameligibility'),
        ),
        migrations.AddField(
            model_name='immunizationprotocolapplied',
            name='authority',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='immunization_protocol_applied_authority', to='base.organizationreference'),
        ),
        migrations.AddField(
            model_name='immunizationprotocolapplied',
            name='target_disease',
            field=models.ManyToManyField(blank=True, related_name='immunization_protocol_applied_usage_disease', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='immunization',
            name='protocol_applied',
            field=models.ManyToManyField(blank=True, related_name='immunization_protocol_applied', to='immunizations.immunizationprotocolapplied'),
        ),
        migrations.AddField(
            model_name='immunizationreaction',
            name='manifestation',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='immunization_reaction_detail', to='observations.observationcodeablereference'),
        ),
        migrations.AddField(
            model_name='immunization',
            name='reaction',
            field=models.ManyToManyField(blank=True, related_name='immunization_reaction', to='immunizations.immunizationreaction'),
        ),
        migrations.AddField(
            model_name='immunizationreasoncodeablereference',
            name='concept',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='immunization_reason_codeable_references_identifier', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='immunization',
            name='reason',
            field=models.ManyToManyField(blank=True, related_name='immunization_reason', to='immunizations.immunizationreasoncodeablereference'),
        ),
        migrations.AddField(
            model_name='immunizationreasonreference',
            name='condition',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='immunization_reason_references_condition', to='conditions.condition'),
        ),
        migrations.AddField(
            model_name='immunizationreasonreference',
            name='diagnostic_report',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='immunization_reason_references_diagnostic_report', to='diagnosticreports.diagnosticreport'),
        ),
        migrations.AddField(
            model_name='immunizationreasonreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='immunization_reason_references_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='immunizationreasonreference',
            name='observation',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='immunization_reason_references_observation', to='observations.observation'),
        ),
        migrations.AddField(
            model_name='immunizationreasoncodeablereference',
            name='reference',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='immunization_reason_codeable_references_reference', to='immunizations.immunizationreasonreference'),
        ),
    ]



"""supply delivery serializers."""

from drf_writable_nested import WritableNestedModelSerializer

from dfhir.base.serializers import (
    BaseReferenceModelSerializer,
    BaseWritableNestedModelSerializer,
    CodeableConceptSerializer,
    IdentifierSerializer,
    PeriodSerializer,
    SimpleQuantitySerializer,
    TimingSerializer,
)
from dfhir.patients.serializers import PatientReferenceSerializer
from dfhir.supplydeliveries.models import (
    SupplyDeliverReference,
    SupplyDelivery,
    SupplyDeliveryContractReference,
    SupplyDeliveryDestinationReference,
    SupplyDeliveryReceiverReference,
    SupplyDeliverySuppliedItem,
    SupplyDeliverySuppliedItemItemReference,
    SupplyDeliverySupplierReference,
)
from dfhir.supplyrequests.serializers import SupplyRequestReferenceSerilizer


class SupplyDeliveryContractReferenceSerializer(BaseReferenceModelSerializer):
    """supply delivery contract reference serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """meta options."""

        model = SupplyDeliveryContractReference
        exclude = ["created_at", "updated_at"]


class SupplyDeliveryReferenceSerializer(BaseReferenceModelSerializer):
    """supply delivery reference serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """meta options."""

        model = SupplyDeliverReference
        exclude = ["created_at", "updated_at"]


class SupplyDeliveryReceiverReferenceSerializer(BaseReferenceModelSerializer):
    """supply delivery receiver reference serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """meta options."""

        model = SupplyDeliveryReceiverReference
        exclude = ["created_at", "updated_at"]


class SupplyDeliverySupplierReferenceSerializer(BaseReferenceModelSerializer):
    """supply delivery supplier reference serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """meta options."""

        model = SupplyDeliverySupplierReference
        exclude = ["created_at", "updated_at"]


class SupplyDeliveryDestinationReferenceSerializer(BaseReferenceModelSerializer):
    """supply delivery destination reference serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """meta options."""

        model = SupplyDeliveryDestinationReference
        exclude = ["created_at", "updated_at"]


class SupplyDeliverySuppliedItemItemReferenceSerializer(BaseReferenceModelSerializer):
    """supply delivery supplied item item reference serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """meta options."""

        model = SupplyDeliverySuppliedItemItemReference
        exclude = ["created_at", "updated_at"]


class SupplyDeliverySuppliedItemSerializer(WritableNestedModelSerializer):
    """supply delivery supplied item serializer."""

    quantity = SimpleQuantitySerializer(required=False)
    condition = CodeableConceptSerializer(required=False)
    item_codeable_concept = CodeableConceptSerializer(required=False)
    item_reference = SupplyDeliverySuppliedItemItemReferenceSerializer(required=False)

    class Meta:
        """meta options."""

        model = SupplyDeliverySuppliedItem
        exclude = ["created_at", "updated_at"]


class SupplyDeliverySerializer(BaseWritableNestedModelSerializer):
    """supply delivery serializer."""

    identifier = IdentifierSerializer(required=False, many=True)
    based_on = SupplyRequestReferenceSerilizer(required=False, many=True)
    part_of = SupplyDeliveryContractReferenceSerializer(required=False, many=True)
    patient = PatientReferenceSerializer(required=False)
    type = CodeableConceptSerializer(required=False)
    stage = CodeableConceptSerializer(required=False)
    supplied_item = SupplyDeliverySuppliedItemSerializer(required=False, many=True)
    occurrence_period = PeriodSerializer(required=False)
    occurrence_timing = TimingSerializer(required=False)
    supplier = SupplyDeliverySupplierReferenceSerializer(required=False)
    destination = SupplyDeliveryDestinationReferenceSerializer(required=False)
    receiver = SupplyDeliveryReceiverReferenceSerializer(required=False, many=True)

    class Meta:
        """meta options."""

        model = SupplyDelivery
        exclude = ["created_at", "updated_at"]



# Generated by Django 5.2 on 2025-05-02 14:24

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('immunizationevaluations', '0002_initial'),
        ('immunizations', '0004_immunizationreference'),
    ]

    operations = [
        migrations.AddField(
            model_name='immunizationevaluation',
            name='immunization_event',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='immunization_evaluation_immunization_event', to='immunizations.immunizationreference'),
        ),
    ]



"""inventory report serializer."""

from drf_writable_nested import WritableNestedModelSerializer

from dfhir.base.serializers import (
    AnnotationSerializer,
    BaseReferenceModelSerializer,
    BaseWritableNestedModelSerializer,
    CodeableConceptSerializer,
    IdentifierSerializer,
    PeriodSerializer,
    QuantitySerializer,
)
from dfhir.inventoryreports.models import (
    InventoryReport,
    InventoryReportInventoryListing,
    InventoryReportInventoryListingItem,
    InventoryReportInventoryListingItemItemCodeableReference,
    InventoryReportInventoryListingItemItemReference,
    InventoryReportReporterReference,
)
from dfhir.locations.serializers import LocationReferenceSerializer


class InventoryReportReporterReferenceSerializer(BaseReferenceModelSerializer):
    """inventory report reporter reference serializer."""

    identifier = IdentifierSerializer(required=True)

    class Meta:
        """meta options."""

        model = InventoryReportReporterReference
        exclude = ["created_at", "updated_at"]


class InventoryReportInventoryListingItemItemReferenceSerializer(
    BaseReferenceModelSerializer
):
    """inventory report listing item item reference serializer."""

    identifier = IdentifierSerializer(required=True)

    class Meta:
        """meta options."""

        model = InventoryReportInventoryListingItemItemReference
        exclude = ["created_at", "updated_at"]


class InventoryReportInventoryListingItemItemCodeableReferenceSerializer(
    WritableNestedModelSerializer
):
    """inventory report listing item item codeable reference serializer."""

    concept = CodeableConceptSerializer(required=False)
    reference = InventoryReportInventoryListingItemItemReferenceSerializer(
        required=False
    )

    class Meta:
        """meta options."""

        model = InventoryReportInventoryListingItemItemCodeableReference
        exclude = ["created_at", "updated_at"]


class InventoryReportInventoryListingItemSerializer(WritableNestedModelSerializer):
    """inventory report inventory listing item serializer."""

    category = CodeableConceptSerializer(required=False)
    quantity = QuantitySerializer(required=False)
    item = InventoryReportInventoryListingItemItemReferenceSerializer(
        required=False, many=True
    )

    class Meta:
        """meta options."""

        model = InventoryReportInventoryListingItem
        exclude = ["created_at", "updated_at"]


class InventoryReportInventoryListingSerializer(WritableNestedModelSerializer):
    """inventory report listing serializer."""

    location = LocationReferenceSerializer(required=False)
    item_status = CodeableConceptSerializer(required=False)
    item = InventoryReportInventoryListingItemItemCodeableReferenceSerializer(
        required=False, many=True
    )

    class Meta:
        """meta options."""

        model = InventoryReportInventoryListing
        exclude = ["created_at", "updated_at"]


class InventoryReportSerializer(BaseWritableNestedModelSerializer):
    """inventory report serializer."""

    identifier = IdentifierSerializer(required=False, many=True)
    operation_type = CodeableConceptSerializer(required=False)
    operation_type_reason = CodeableConceptSerializer(required=False)
    reporter = InventoryReportReporterReferenceSerializer(required=False)
    reporting_period = PeriodSerializer(required=False)
    inventory_listing = InventoryReportInventoryListingSerializer(
        required=False, many=True
    )
    note = AnnotationSerializer(many=True, required=False)

    class Meta:
        """meta options."""

        model = InventoryReport
        exclude = ["created_at", "updated_at"]



"""Serializers for persons app."""

from drf_writable_nested import WritableNestedModelSerializer

from dfhir.base.serializers import (
    AddressSerializer,
    AttachmentSerializer,
    BaseReferenceModelSerializer,
    BaseWritableNestedModelSerializer,
    CodeableConceptSerializer,
    CommunicationSerializer,
    ContactPointSerializer,
    HumanNameSerializer,
    IdentifierSerializer,
    OrganizationReferenceSerializer,
)

from .models import (
    Person,
    PersonLink,
    PersonLinkTargetReference,
    PersonReference,
)


class PersonLinkTargetReferenceSerializer(BaseReferenceModelSerializer):
    """PersonLinkTargetReferenceSerializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """Meta class for PersonLinkTargetReferenceSerializer."""

        model = PersonLinkTargetReference
        exclude = ["created_at", "updated_at"]


class PersonLinkSerializer(WritableNestedModelSerializer):
    """PersonLinkSerializer."""

    target = PersonLinkTargetReferenceSerializer(required=False)

    class Meta:
        """Meta class for PersonLinkSerializer."""

        model = PersonLink
        exclude = ["created_at", "updated_at"]


class PersonReferenceSerializer(BaseReferenceModelSerializer):
    """PersonReferenceSerializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """Meta class for PersonReferenceSerializer."""

        model = PersonReference
        exclude = ["created_at", "updated_at"]


class PersonSerializer(BaseWritableNestedModelSerializer):
    """PersonSerializer."""

    identifier = IdentifierSerializer(many=True, required=False)
    name = HumanNameSerializer(required=False, many=True)
    telecom = ContactPointSerializer(required=False, many=True)
    address = AddressSerializer(required=False, many=True)
    marital_status = CodeableConceptSerializer(required=False)
    photo = AttachmentSerializer(required=False, many=True)
    communication = CommunicationSerializer(required=False, many=True)
    managing_organization = OrganizationReferenceSerializer(required=False)
    link = PersonLinkSerializer(required=False, many=True)

    class Meta:
        """Meta class for PersonSerializer."""

        model = Person
        exclude = ["created_at", "updated_at"]



"""immunization serializers."""

from dfhir.base.serializers import (
    BaseWritableNestedModelSerializer,
    CodeableConceptSerializer,
    IdentifierSerializer,
    OrganizationReferenceSerializer,
)
from dfhir.immunizationevaluations.models import ImmunizationEvaluation
from dfhir.immunizations.serializers import ImmunizationReferenceSerializer
from dfhir.patients.serializers import PatientReferenceSerializer


class ImmunizationEvaluationSerializer(BaseWritableNestedModelSerializer):
    """immunization evaluation serializer."""

    identifier = IdentifierSerializer(many=True, required=False)
    patient = PatientReferenceSerializer(many=False, required=False)
    authority = OrganizationReferenceSerializer(many=False, required=False)
    target_disease = CodeableConceptSerializer(many=False, required=False)
    immunization_event = ImmunizationReferenceSerializer(required=False)
    dose_status = CodeableConceptSerializer(many=False, required=False)
    dose_status_reason = CodeableConceptSerializer(many=True, required=False)

    class Meta:
        """meta class."""

        model = ImmunizationEvaluation
        exclude = ["created_at", "updated_at"]



"""supply delivery views."""

from django.http import Http404
from drf_spectacular.utils import extend_schema
from rest_framework import status
from rest_framework.permissions import AllowAny
from rest_framework.response import Response
from rest_framework.views import APIView

from dfhir.supplydeliveries.models import SupplyDelivery
from dfhir.supplydeliveries.serializers import SupplyDeliverySerializer


class SupplyDeliveryListView(APIView):
    """supply delivery list view."""

    permission_classes = (AllowAny,)

    @extend_schema(responses={200: SupplyDeliverySerializer(many=True)})
    def get(self, request):
        """Get supply delivery objects."""
        supply_deliveries = SupplyDeliverySerializer(SupplyDelivery.objects.all())
        serializer = SupplyDeliverySerializer(supply_deliveries, many=True)
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(
        request=SupplyDeliverySerializer, responses={201: SupplyDeliverySerializer}
    )
    def post(self, request):
        """Create supply delivery object."""
        serializer = SupplyDeliverySerializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_201_CREATED)


class SupplyDeliveryDetailedView(APIView):
    """supply delivery detailed view."""

    def get_object(self, pk):
        """Get object method."""
        try:
            return SupplyDelivery.objects.get(pk=pk)
        except SupplyDelivery.DoesNotExist as err:
            raise Http404 from err

    @extend_schema(responses={200: SupplyDeliverySerializer})
    def get(self, request, pk):
        """Get a supply delivery item."""
        supply_delivery = self.get_object(pk)
        serializer = SupplyDeliverySerializer(supply_delivery)
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(
        request=SupplyDeliverySerializer, responses={200: SupplyDeliverySerializer}
    )
    def patch(self, request, pk):
        """Update a supply delivery item."""
        supply_delivery = self.get_object(pk)
        serializer = SupplyDeliverySerializer(
            supply_delivery, data=request.data, partial=True
        )
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(responses={204: None})
    def delete(self, request, pk):
        """Delete supply delivery item."""
        supply_delivery = self.get_object(pk)
        supply_delivery.delete()
        return Response(status=status.HTTP_204_NO_CONTENT)



"""CoverageEligibilityResponse status choices."""

from django.db import models
from django.utils.translation import gettext_lazy as _


class CoverageEligibilityResponseStatus(models.TextChoices):
    """CoverageEligibilityResponse status code choices."""

    ACTIVE = "active", _("Active")
    CANCELLED = "cancelled", _("Cancelled")
    DRAFT = "draft", _("Draft")
    ENTERED_IN_ERROR = "entered-in-error", _("Entered in Error")


class CoverageEligibilityResponsePurpose(models.TextChoices):
    """CoverageEligibilityResponse purpose code choices."""

    AUTH_REQUIREMENTS = "auth-requirements", _("Auth Requirements")
    BENEFITS = "benefits", _("Benefits")
    DISCOVERY = "discovery", _("Discovery")
    VALIDATION = "validation", _("Validation")


class CoverageEligibilityResponseOutcome(models.TextChoices):
    """CoverageEligibilityResponse outcome code choices."""

    QUEUED = "queued", _("Queued")
    COMPLETE = "complete", _("Complete")
    ERROR = "error", _("Error")
    PARTIAL = "partial", _("Partial")



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0005_initial'),
        ('coverageeligibilityresponses', '0002_initial'),
        ('practitionerroles', '0001_initial'),
        ('practitioners', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='coverageeligibilityresponseinsuranceitem',
            name='provider',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='coverage_eligibility_response_insurance_item_provider', to='practitioners.practitionerpractitionerrolereference'),
        ),
        migrations.AddField(
            model_name='coverageeligibilityresponseinsuranceitem',
            name='term',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='coverage_eligibility_response_insurance_item_term', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='coverageeligibilityresponseinsuranceitem',
            name='unit',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='coverage_eligibility_response_insurance_item_unit_price', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='coverageeligibilityresponseinsurance',
            name='item',
            field=models.ManyToManyField(blank=True, related_name='coverage_eligibility_response_insurance_item', to='coverageeligibilityresponses.coverageeligibilityresponseinsuranceitem'),
        ),
        migrations.AddField(
            model_name='coverageeligibilityresponseinsuranceitembenefit',
            name='allowed_money',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='coverage_eligibility_response_insurance_item_benefit_allowed_money', to='base.money'),
        ),
        migrations.AddField(
            model_name='coverageeligibilityresponseinsuranceitembenefit',
            name='type',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='coverage_eligibility_response_insurance_item_benefit_type', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='coverageeligibilityresponseinsuranceitembenefit',
            name='used_money',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='coverage_eligibility_response_insurance_item_benefit_used_money', to='base.money'),
        ),
        migrations.AddField(
            model_name='coverageeligibilityresponseinsuranceitem',
            name='benefit',
            field=models.ManyToManyField(blank=True, related_name='coverage_eligibility_response_insurance_item_benefit', to='coverageeligibilityresponses.coverageeligibilityresponseinsuranceitembenefit'),
        ),
        migrations.AddField(
            model_name='coverageeligibilityresponserequesterreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='coverage_eligibility_response_requester_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='coverageeligibilityresponserequesterreference',
            name='organization',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='coverage_eligibility_response_requester_reference_organization', to='base.organizationreference'),
        ),
        migrations.AddField(
            model_name='coverageeligibilityresponserequesterreference',
            name='practitioner',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='coverage_eligibility_response_requester_reference_practitioner', to='practitioners.practitioner'),
        ),
        migrations.AddField(
            model_name='coverageeligibilityresponserequesterreference',
            name='practitioner_role',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='coverage_eligibility_response_requester_reference_practitioner_role', to='practitionerroles.practitionerrole'),
        ),
        migrations.AddField(
            model_name='coverageeligibilityresponse',
            name='requestor',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='coverage_eligibility_response_requestor', to='coverageeligibilityresponses.coverageeligibilityresponserequesterreference'),
        ),
    ]



"""Base validators for the dfhir app."""

from django.utils import timezone
from rest_framework import serializers


def validate_date_time_fields(start_date_time, end_date_time):
    """Validate date time fields."""
    if start_date_time and end_date_time and start_date_time > end_date_time:
        raise serializers.ValidationError(
            {"start_date_time": "Start date time must be before end date time"}
        )
    if start_date_time and start_date_time < timezone.now():
        raise serializers.ValidationError(
            {"start_date_time": "Start date time must be in the future"}
        )
    if end_date_time and end_date_time < timezone.now():
        raise serializers.ValidationError(
            {"end_date_time": "End date time must be in the future"}
        )



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0005_initial'),
        ('careteams', '0004_initial'),
        ('conditions', '0004_initial'),
        ('contracts', '0002_initial'),
        ('devices', '0001_initial'),
        ('diagnosticreports', '0002_initial'),
        ('documentreferences', '0003_initial'),
        ('encounters', '0003_initial'),
        ('groups', '0001_initial'),
        ('locations', '0001_initial'),
        ('observations', '0002_initial'),
        ('organizations', '0001_initial'),
        ('patients', '0001_initial'),
        ('practitionerroles', '0001_initial'),
        ('practitioners', '0001_initial'),
        ('relatedpersons', '0001_initial'),
        ('substances', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='contractauthorreference',
            name='practitioner',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_author_reference_practitioner', to='practitioners.practitioner'),
        ),
        migrations.AddField(
            model_name='contractauthorreference',
            name='practitioner_role',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_author_reference_practitioner_role', to='practitionerroles.practitionerrole'),
        ),
        migrations.AddField(
            model_name='contract',
            name='author',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_author', to='contracts.contractauthorreference'),
        ),
        migrations.AddField(
            model_name='contractcontentdefinition',
            name='sub_type',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_content_definition_sub_type', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='contractcontentdefinition',
            name='type',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_content_definition_type', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='contract',
            name='content_definition',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_content_definition', to='contracts.contractcontentdefinition'),
        ),
        migrations.AddField(
            model_name='contractcontentdefinitionpublisherreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_content_definition_publisher_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='contractcontentdefinitionpublisherreference',
            name='organization',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_content_definition_publisher_reference_organization', to='organizations.organization'),
        ),
        migrations.AddField(
            model_name='contractcontentdefinitionpublisherreference',
            name='practitioner',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_content_definition_publisher_reference_practitioner', to='practitioners.practitioner'),
        ),
        migrations.AddField(
            model_name='contractcontentdefinitionpublisherreference',
            name='practitioner_role',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_content_definition_publisher_reference_practitioner_role', to='practitionerroles.practitionerrole'),
        ),
        migrations.AddField(
            model_name='contractcontentdefinition',
            name='publisher',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_content_definition_publisher', to='contracts.contractcontentdefinitionpublisherreference'),
        ),
        migrations.AddField(
            model_name='contractfriendly',
            name='content_attachment',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_friendly_content_attachment', to='base.attachment'),
        ),
        migrations.AddField(
            model_name='contract',
            name='friendly',
            field=models.ManyToManyField(blank=True, related_name='contract_friendly', to='contracts.contractfriendly'),
        ),
        migrations.AddField(
            model_name='contractfriendlycontentreferencereference',
            name='document_reference',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_friendly_content_reference_reference_document_reference', to='documentreferences.documentreference'),
        ),
        migrations.AddField(
            model_name='contractfriendlycontentreferencereference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_friendly_content_reference_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='contractfriendly',
            name='content_reference',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_friendly_content_reference', to='contracts.contractfriendlycontentreferencereference'),
        ),
        migrations.AddField(
            model_name='contractlegal',
            name='content_attachment',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_legal_content_attachment', to='base.attachment'),
        ),
        migrations.AddField(
            model_name='contract',
            name='legal',
            field=models.ManyToManyField(blank=True, related_name='contract_legal', to='contracts.contractlegal'),
        ),
        migrations.AddField(
            model_name='contractlegalcontentreferencereference',
            name='document_reference',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_legal_content_reference_reference_document_reference', to='documentreferences.documentreference'),
        ),
        migrations.AddField(
            model_name='contractlegalcontentreferencereference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_legal_content_reference_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='contractlegal',
            name='content_reference',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_legal_content_reference', to='contracts.contractlegalcontentreferencereference'),
        ),
        migrations.AddField(
            model_name='contractlegallybindingreferencereference',
            name='contract',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_legally_binding_reference_reference_contract', to='contracts.contract'),
        ),
        migrations.AddField(
            model_name='contractlegallybindingreferencereference',
            name='document_reference',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_legally_binding_reference_reference_document_reference', to='documentreferences.documentreference'),
        ),
        migrations.AddField(
            model_name='contractlegallybindingreferencereference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_legally_binding_reference_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='contract',
            name='legally_binding_reference',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_legally_binding_reference', to='contracts.contractlegallybindingreferencereference'),
        ),
        migrations.AddField(
            model_name='contractreference',
            name='contract',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_reference_contract', to='contracts.contract'),
        ),
        migrations.AddField(
            model_name='contractreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='contract',
            name='instantiates_canonical',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_instantiates_canonical', to='contracts.contractreference'),
        ),
        migrations.AddField(
            model_name='contractrule',
            name='content_attachment',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_rule_content_attachment', to='base.attachment'),
        ),
        migrations.AddField(
            model_name='contract',
            name='rule',
            field=models.ManyToManyField(blank=True, related_name='contract_rule', to='contracts.contractrule'),
        ),
        migrations.AddField(
            model_name='contractsigner',
            name='signature',
            field=models.ManyToManyField(blank=True, related_name='contract_signer_signature', to='base.signature'),
        ),
        migrations.AddField(
            model_name='contractsigner',
            name='type',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_signer_type', to='base.coding'),
        ),
        migrations.AddField(
            model_name='contract',
            name='signer',
            field=models.ManyToManyField(blank=True, related_name='contract_signer', to='contracts.contractsigner'),
        ),
        migrations.AddField(
            model_name='contractsignerpartyreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_signer_party_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='contractsignerpartyreference',
            name='organization',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_signer_party_reference_organization', to='organizations.organization'),
        ),
        migrations.AddField(
            model_name='contractsignerpartyreference',
            name='patient',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_signer_party_reference_patient', to='patients.patient'),
        ),
        migrations.AddField(
            model_name='contractsignerpartyreference',
            name='practitioner',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_signer_party_reference_practitioner', to='practitioners.practitioner'),
        ),
        migrations.AddField(
            model_name='contractsignerpartyreference',
            name='practitioner_role',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_signer_party_reference_practitioner_role', to='practitionerroles.practitionerrole'),
        ),
        migrations.AddField(
            model_name='contractsignerpartyreference',
            name='related_person',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_signer_party_reference_related_person', to='relatedpersons.relatedperson'),
        ),
        migrations.AddField(
            model_name='contractsigner',
            name='party',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_signer_party', to='contracts.contractsignerpartyreference'),
        ),
        migrations.AddField(
            model_name='contractsubjectreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='subject_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='contractsubjectreference',
            name='patient',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='subject_reference_patient', to='patients.patient'),
        ),
        migrations.AddField(
            model_name='contract',
            name='subject',
            field=models.ManyToManyField(blank=True, related_name='contract_subject', to='contracts.contractsubjectreference'),
        ),
        migrations.AddField(
            model_name='contractterm',
            name='applies',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_term_applies', to='base.period'),
        ),
        migrations.AddField(
            model_name='contractterm',
            name='group',
            field=models.ManyToManyField(blank=True, to='contracts.contractterm'),
        ),
        migrations.AddField(
            model_name='contractterm',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_term_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='contractterm',
            name='sub_type',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_term_sub_type', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='contractterm',
            name='topic_codeable_concept',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_term_topic_codeable_concept', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='contractterm',
            name='topic_reference',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_term_topic_reference', to='base.reference'),
        ),
        migrations.AddField(
            model_name='contractterm',
            name='type',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_term_type', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='contract',
            name='term',
            field=models.ManyToManyField(blank=True, related_name='contract_term', to='contracts.contractterm'),
        ),
        migrations.AddField(
            model_name='contracttermaction',
            name='context',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_term_action_context', to='encounters.encounterepisodeofcarereference'),
        ),
        migrations.AddField(
            model_name='contracttermaction',
            name='intent',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_term_action_intent', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='contracttermaction',
            name='note',
            field=models.ManyToManyField(blank=True, related_name='contract_term_action_note', to='base.annotation'),
        ),
        migrations.AddField(
            model_name='contracttermaction',
            name='occurrence_period',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_term_action_occurrence_period', to='base.period'),
        ),
        migrations.AddField(
            model_name='contracttermaction',
            name='occurrence_timing',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_term_action_occurrence_timing', to='base.timing'),
        ),
        migrations.AddField(
            model_name='contracttermaction',
            name='performer_role',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_term_action_performer_role', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='contracttermaction',
            name='performer_type',
            field=models.ManyToManyField(blank=True, related_name='contract_term_action_performer_type', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='contracttermaction',
            name='status',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_term_action_status', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='contracttermaction',
            name='type',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_term_action_type', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='contractterm',
            name='action',
            field=models.ManyToManyField(blank=True, related_name='contract_term_action', to='contracts.contracttermaction'),
        ),
        migrations.AddField(
            model_name='contracttermactionperformerreference',
            name='care_team',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_term_action_performer_reference_care_team', to='careteams.careteam'),
        ),
        migrations.AddField(
            model_name='contracttermactionperformerreference',
            name='device',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_term_action_performer_reference_device', to='devices.device'),
        ),
        migrations.AddField(
            model_name='contracttermactionperformerreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_term_action_performer_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='contracttermactionperformerreference',
            name='location',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_term_action_performer_reference_location', to='locations.location'),
        ),
        migrations.AddField(
            model_name='contracttermactionperformerreference',
            name='organization',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_term_action_performer_reference_organization', to='organizations.organization'),
        ),
        migrations.AddField(
            model_name='contracttermactionperformerreference',
            name='patient',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_term_action_performer_reference_patient', to='patients.patient'),
        ),
        migrations.AddField(
            model_name='contracttermactionperformerreference',
            name='practitioner',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_term_action_performer_reference_practitioner', to='practitioners.practitioner'),
        ),
        migrations.AddField(
            model_name='contracttermactionperformerreference',
            name='practitioner_role',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_term_action_performer_reference_practitioner_role', to='practitionerroles.practitionerrole'),
        ),
        migrations.AddField(
            model_name='contracttermactionperformerreference',
            name='related_person',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_term_action_performer_reference_related_person', to='relatedpersons.relatedperson'),
        ),
        migrations.AddField(
            model_name='contracttermactionperformerreference',
            name='substance',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_term_action_performer_reference_substance', to='substances.substance'),
        ),
        migrations.AddField(
            model_name='contracttermaction',
            name='performer',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_term_action_performer', to='contracts.contracttermactionperformerreference'),
        ),
        migrations.AddField(
            model_name='contracttermactionreasoncodeablereference',
            name='concept',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_term_action_reason_codeable_reference_concept', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='contracttermaction',
            name='reason',
            field=models.ManyToManyField(blank=True, related_name='contract_term_action_reason', to='contracts.contracttermactionreasoncodeablereference'),
        ),
        migrations.AddField(
            model_name='contracttermactionreasonreference',
            name='condition',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_term_action_reason_reference_condition', to='conditions.condition'),
        ),
        migrations.AddField(
            model_name='contracttermactionreasonreference',
            name='diagnostic_report',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_term_action_reason_reference_diagnostic_report', to='diagnosticreports.diagnosticreport'),
        ),
        migrations.AddField(
            model_name='contracttermactionreasonreference',
            name='document_reference',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_term_action_reason_reference_document_reference', to='documentreferences.documentreference'),
        ),
        migrations.AddField(
            model_name='contracttermactionreasonreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_term_action_reason_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='contracttermactionreasonreference',
            name='observation',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_term_action_reason_reference_observation', to='observations.observation'),
        ),
        migrations.AddField(
            model_name='contracttermactionreasoncodeablereference',
            name='reference',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_term_action_reason_codeable_reference_reference', to='contracts.contracttermactionreasonreference'),
        ),
        migrations.AddField(
            model_name='contracttermactionrequesterreference',
            name='device',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_term_action_requester_reference_device', to='devices.device'),
        ),
        migrations.AddField(
            model_name='contracttermactionrequesterreference',
            name='group',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_term_action_requester_reference_group', to='groups.group'),
        ),
        migrations.AddField(
            model_name='contracttermactionrequesterreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_term_action_requester_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='contracttermactionrequesterreference',
            name='organization',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_term_action_requester_reference_organization', to='organizations.organization'),
        ),
        migrations.AddField(
            model_name='contracttermactionrequesterreference',
            name='patient',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_term_action_requester_reference_patient', to='patients.patient'),
        ),
        migrations.AddField(
            model_name='contracttermactionrequesterreference',
            name='practitioner',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_term_action_requester_reference_practitioner', to='practitioners.practitioner'),
        ),
        migrations.AddField(
            model_name='contracttermactionrequesterreference',
            name='practitioner_role',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_term_action_requester_reference_practitioner_role', to='practitionerroles.practitionerrole'),
        ),
        migrations.AddField(
            model_name='contracttermactionrequesterreference',
            name='related_person',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_term_action_requester_reference_related_person', to='relatedpersons.relatedperson'),
        ),
        migrations.AddField(
            model_name='contracttermaction',
            name='requester',
            field=models.ManyToManyField(blank=True, related_name='contract_term_action_requester', to='contracts.contracttermactionrequesterreference'),
        ),
        migrations.AddField(
            model_name='contracttermactionsubject',
            name='role',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_term_action_subject_role', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='contracttermaction',
            name='subject',
            field=models.ManyToManyField(blank=True, related_name='contract_term_action_subject', to='contracts.contracttermactionsubject'),
        ),
        migrations.AddField(
            model_name='contracttermactionsubjectreferencereference',
            name='device',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_term_action_subject_reference_reference_device', to='devices.device'),
        ),
        migrations.AddField(
            model_name='contracttermactionsubjectreferencereference',
            name='group',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_term_action_subject_reference_reference_group', to='groups.group'),
        ),
        migrations.AddField(
            model_name='contracttermactionsubjectreferencereference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_term_action_subject_reference_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='contracttermactionsubjectreferencereference',
            name='organization',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_term_action_subject_reference_reference_organization', to='organizations.organization'),
        ),
        migrations.AddField(
            model_name='contracttermactionsubjectreferencereference',
            name='patient',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_term_action_subject_reference_reference_patient', to='patients.patient'),
        ),
        migrations.AddField(
            model_name='contracttermactionsubjectreferencereference',
            name='practitioner',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_term_action_subject_reference_reference_practitioner', to='practitioners.practitioner'),
        ),
        migrations.AddField(
            model_name='contracttermactionsubjectreferencereference',
            name='practitioner_role',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_term_action_subject_reference_reference_practitioner_role', to='practitionerroles.practitionerrole'),
        ),
        migrations.AddField(
            model_name='contracttermactionsubjectreferencereference',
            name='related_person',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_term_action_subject_reference_reference_related_person', to='relatedpersons.relatedperson'),
        ),
        migrations.AddField(
            model_name='contracttermactionsubject',
            name='reference',
            field=models.ManyToManyField(blank=True, related_name='contract_term_action_subject_reference', to='contracts.contracttermactionsubjectreferencereference'),
        ),
        migrations.AddField(
            model_name='contracttermasset',
            name='period',
            field=models.ManyToManyField(blank=True, related_name='contract_term_asset_period', to='base.period'),
        ),
        migrations.AddField(
            model_name='contracttermasset',
            name='period_type',
            field=models.ManyToManyField(blank=True, related_name='contract_term_asset_period_type', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='contracttermasset',
            name='relationship',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_term_asset_relationship', to='base.coding'),
        ),
        migrations.AddField(
            model_name='contracttermasset',
            name='scope',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_term_asset_scope', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='contracttermasset',
            name='subtype',
            field=models.ManyToManyField(blank=True, related_name='contract_term_asset_subtype', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='contracttermasset',
            name='type',
            field=models.ManyToManyField(blank=True, related_name='contract_term_asset_type', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='contracttermasset',
            name='type_reference',
            field=models.ManyToManyField(blank=True, related_name='contract_term_asset_type_reference', to='base.reference'),
        ),
        migrations.AddField(
            model_name='contracttermasset',
            name='use_period',
            field=models.ManyToManyField(blank=True, related_name='contract_term_asset_use_period', to='base.period'),
        ),
        migrations.AddField(
            model_name='contractterm',
            name='asset',
            field=models.ManyToManyField(blank=True, related_name='contract_term_asset', to='contracts.contracttermasset'),
        ),
        migrations.AddField(
            model_name='contracttermassetcontext',
            name='code',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_term_asset_context_code', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='contracttermassetcontext',
            name='reference',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_term_asset_context_reference', to='base.reference'),
        ),
        migrations.AddField(
            model_name='contracttermasset',
            name='context',
            field=models.ManyToManyField(blank=True, related_name='contract_term_asset_context', to='contracts.contracttermassetcontext'),
        ),
        migrations.AddField(
            model_name='contracttermassetvalueditem',
            name='entity_codeable_concept',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_term_asset_valued_item_entity_codeable_concept', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='contracttermassetvalueditem',
            name='entity_reference',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_term_asset_valued_item_entity_reference', to='base.reference'),
        ),
        migrations.AddField(
            model_name='contracttermassetvalueditem',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_term_asset_valued_item_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='contracttermassetvalueditem',
            name='net',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_term_asset_valued_item_net', to='base.money'),
        ),
        migrations.AddField(
            model_name='contracttermassetvalueditem',
            name='quantity',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_term_asset_valued_item_quantity', to='base.simplequantity'),
        ),
        migrations.AddField(
            model_name='contracttermassetvalueditem',
            name='unit_price',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_term_asset_valued_item_unit_price', to='base.money'),
        ),
        migrations.AddField(
            model_name='contracttermasset',
            name='valued_item',
            field=models.ManyToManyField(blank=True, related_name='contract_term_asset_valued_item', to='contracts.contracttermassetvalueditem'),
        ),
        migrations.AddField(
            model_name='contracttermassetvalueditemrecipientreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_tem_asset_valued_item_recipient_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='contracttermassetvalueditemrecipientreference',
            name='organization',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_tem_asset_valued_item_recipient_organization', to='organizations.organization'),
        ),
        migrations.AddField(
            model_name='contracttermassetvalueditemrecipientreference',
            name='patient',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_tem_asset_valued_item_recipient_patient', to='patients.patient'),
        ),
        migrations.AddField(
            model_name='contracttermassetvalueditemrecipientreference',
            name='practitioner',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_tem_asset_valued_item_recipient_practitioner', to='practitioners.practitioner'),
        ),
        migrations.AddField(
            model_name='contracttermassetvalueditemrecipientreference',
            name='practitioner_role',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_tem_asset_valued_item_recipient_practitioner_role', to='practitionerroles.practitionerrole'),
        ),
        migrations.AddField(
            model_name='contracttermassetvalueditemrecipientreference',
            name='related_person',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_tem_asset_valued_item_recipient_related_person', to='relatedpersons.relatedperson'),
        ),
        migrations.AddField(
            model_name='contracttermassetvalueditem',
            name='recipient',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_term_asset_valued_item_recipient', to='contracts.contracttermassetvalueditemrecipientreference'),
        ),
        migrations.AddField(
            model_name='contracttermassetvalueditemresponsiblereference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_tem_asset_valued_item_responsible_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='contracttermassetvalueditemresponsiblereference',
            name='organization',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_tem_asset_valued_item_responsible_organization', to='organizations.organization'),
        ),
        migrations.AddField(
            model_name='contracttermassetvalueditemresponsiblereference',
            name='patient',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_tem_asset_valued_item_responsible_patient', to='patients.patient'),
        ),
        migrations.AddField(
            model_name='contracttermassetvalueditemresponsiblereference',
            name='practitioner',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_tem_asset_valued_item_responsible_practitioner', to='practitioners.practitioner'),
        ),
        migrations.AddField(
            model_name='contracttermassetvalueditemresponsiblereference',
            name='practitioner_role',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_tem_asset_valued_item_responsible_practitioner_role', to='practitionerroles.practitionerrole'),
        ),
        migrations.AddField(
            model_name='contracttermassetvalueditemresponsiblereference',
            name='related_person',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_tem_asset_valued_item_responsible_related_person', to='relatedpersons.relatedperson'),
        ),
        migrations.AddField(
            model_name='contracttermassetvalueditem',
            name='responsible',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_term_asset_valued_item_responsible', to='contracts.contracttermassetvalueditemresponsiblereference'),
        ),
        migrations.AddField(
            model_name='contracttermoffer',
            name='decision',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_term_offer_decision', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='contracttermoffer',
            name='decision_mode',
            field=models.ManyToManyField(blank=True, related_name='contract_term_offer_decision_mode', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='contracttermoffer',
            name='identifier',
            field=models.ManyToManyField(blank=True, related_name='contract_term_offer_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='contracttermoffer',
            name='topic',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_term_offer_topic', to='base.reference'),
        ),
        migrations.AddField(
            model_name='contracttermoffer',
            name='type',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_term_offer_type', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='contractterm',
            name='offer',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_term_offer', to='contracts.contracttermoffer'),
        ),
        migrations.AddField(
            model_name='contracttermofferanswer',
            name='value_attachment',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_term_offer_answer_attachment', to='base.attachment'),
        ),
        migrations.AddField(
            model_name='contracttermofferanswer',
            name='value_coding',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_term_offer_answer_coding', to='base.coding'),
        ),
        migrations.AddField(
            model_name='contracttermofferanswer',
            name='value_quantity',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_term_offer_answer_quantity', to='base.quantity'),
        ),
        migrations.AddField(
            model_name='contracttermofferanswer',
            name='value_reference',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_term_offer_answer_reference', to='base.reference'),
        ),
        migrations.AddField(
            model_name='contracttermoffer',
            name='answer',
            field=models.ManyToManyField(blank=True, related_name='contract_term_offer_answer', to='contracts.contracttermofferanswer'),
        ),
        migrations.AddField(
            model_name='contracttermasset',
            name='answer',
            field=models.ManyToManyField(blank=True, related_name='contract_term_asset_answer', to='contracts.contracttermofferanswer'),
        ),
        migrations.AddField(
            model_name='contracttermofferparty',
            name='role',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_term_offer_party_role', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='contracttermoffer',
            name='party',
            field=models.ManyToManyField(blank=True, related_name='contract_term_offer_party', to='contracts.contracttermofferparty'),
        ),
        migrations.AddField(
            model_name='contracttermofferpartyreferencereference',
            name='device',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_term_offer_party_reference_device', to='devices.device'),
        ),
        migrations.AddField(
            model_name='contracttermofferpartyreferencereference',
            name='group',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_term_offer_party_reference_group', to='groups.group'),
        ),
        migrations.AddField(
            model_name='contracttermofferpartyreferencereference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_term_offer_party_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='contracttermofferpartyreferencereference',
            name='organization',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_term_offer_party_reference_organization', to='organizations.organization'),
        ),
        migrations.AddField(
            model_name='contracttermofferpartyreferencereference',
            name='patient',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_term_offer_party_reference_patient', to='patients.patient'),
        ),
        migrations.AddField(
            model_name='contracttermofferpartyreferencereference',
            name='practitioner',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_term_offer_party_reference_practitioner', to='practitioners.practitioner'),
        ),
        migrations.AddField(
            model_name='contracttermofferpartyreferencereference',
            name='practitioner_role',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_term_offer_party_reference_practitioner_role', to='practitionerroles.practitionerrole'),
        ),
        migrations.AddField(
            model_name='contracttermofferpartyreferencereference',
            name='related_person',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_term_offer_party_reference_related_person', to='relatedpersons.relatedperson'),
        ),
        migrations.AddField(
            model_name='contracttermofferparty',
            name='reference',
            field=models.ManyToManyField(blank=True, related_name='contract_term_offer_party_reference', to='contracts.contracttermofferpartyreferencereference'),
        ),
        migrations.AddField(
            model_name='contracttermsecuritylabel',
            name='category',
            field=models.ManyToManyField(blank=True, related_name='contract_term_security_label_category', to='base.coding'),
        ),
        migrations.AddField(
            model_name='contracttermsecuritylabel',
            name='classification',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='contract_term_security_label_classification', to='base.coding'),
        ),
        migrations.AddField(
            model_name='contracttermsecuritylabel',
            name='control',
            field=models.ManyToManyField(blank=True, related_name='contract_term_security_label_control', to='base.coding'),
        ),
        migrations.AddField(
            model_name='contractterm',
            name='security_label',
            field=models.ManyToManyField(blank=True, related_name='contract_term_security_label', to='contracts.contracttermsecuritylabel'),
        ),
    ]



"""Invoices views."""

from django.http import Http404
from drf_spectacular.utils import extend_schema
from rest_framework import status
from rest_framework.permissions import AllowAny
from rest_framework.response import Response
from rest_framework.views import APIView

from .models import Invoice
from .serializers import InvoiceSerializer


class InvoiceListView(APIView):
    """Invoice list view."""

    permission_classes = [AllowAny]

    @extend_schema(responses={200: InvoiceSerializer(many=True)})
    def get(self, request):
        """Get a list of invoices."""
        invoices = Invoice.objects.all()
        serializer = InvoiceSerializer(invoices, many=True)
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(request=InvoiceSerializer, responses={201: InvoiceSerializer})
    def post(self, request):
        """Create an invoice."""
        serializer = InvoiceSerializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_201_CREATED)


class InvoiceDetailView(APIView):
    """Invoice detail view."""

    permission_classes = [AllowAny]

    def get_object(self, pk):
        """Get an invoice object."""
        try:
            return Invoice.objects.get(pk=pk)
        except Invoice.DoesNotExist as err:
            raise Http404 from err

    @extend_schema(responses={200: InvoiceSerializer})
    def get(self, request, pk=None):
        """Get an invoice."""
        invoice = self.get_object(pk)
        serializer = InvoiceSerializer(invoice)
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(request=InvoiceSerializer, responses={200: InvoiceSerializer})
    def patch(self, request, pk=None):
        """Update an invoice."""
        invoice = self.get_object(pk)
        serializer = InvoiceSerializer(invoice, data=request.data, partial=True)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(responses={204: None})
    def delete(self, request, pk=None):
        """Delete an invoice."""
        invoice = self.get_object(pk)
        invoice.delete()
        return Response(status=status.HTTP_204_NO_CONTENT)



"""practitioner role serializers."""

from rest_framework import serializers

from dfhir.base.serializers import (
    AvailabilitySerializer,
    BaseReferenceModelSerializer,
    BaseWritableNestedModelSerializer,
    CodeableConceptSerializer,
    ExtendedContactDetailSerializer,
    IdentifierSerializer,
    PeriodSerializer,
)
from dfhir.endpoints.serializers import EndpointReferenceSerializer
from dfhir.healthcareservices.serializers import HealthCareServiceReferenceSerializer
from dfhir.locations.serializers import LocationReferenceSerializer
from dfhir.organizations.serializers import OrganizationReferenceSerializer
from dfhir.practitionerroles.models import (
    PractitionerRole,
    PractitionerRoleCode,
    PractitionerRoleReference,
)
from dfhir.practitioners.serializers import (
    PractitionerReferenceSerializer,
)


class PractitionerRoleCodeSerializer(serializers.ModelSerializer):
    """Practitioner Role Code serializer."""

    class Meta:
        """Meta class."""

        model = PractitionerRoleCode
        exclude = ["created_at", "updated_at"]


# Practitioner role
class PractitionerRoleSerializer(BaseWritableNestedModelSerializer):
    """Practitioner Role serializer."""

    identifier = IdentifierSerializer(many=True, required=False)
    period = PeriodSerializer(many=False, required=False)
    practitioner = PractitionerReferenceSerializer(many=False, required=False)
    organization = OrganizationReferenceSerializer(many=False, required=False)
    network = OrganizationReferenceSerializer(many=True, required=False)
    code = CodeableConceptSerializer(many=True, required=False)
    specialty = CodeableConceptSerializer(many=True, required=False)
    location = LocationReferenceSerializer(many=True, required=False)
    healthcare_service = HealthCareServiceReferenceSerializer(many=True, required=False)
    characteristic = CodeableConceptSerializer(many=True, required=False)
    availability = AvailabilitySerializer(many=False, required=False)
    communication = CodeableConceptSerializer(many=True, required=False)
    contact = ExtendedContactDetailSerializer(many=True, required=False)
    endpoint = EndpointReferenceSerializer(many=True, required=False)

    class Meta:
        """Meta class."""

        model = PractitionerRole
        exclude = ["created_at", "updated_at"]


class PractitionerRoleWithPractitionerIdSerializer(PractitionerRoleSerializer):
    """Practitioner Role with Practitioner ID serializer."""

    class Meta:
        """Meta class."""

        model = PractitionerRole
        exclude = ["created_at", "updated_at"]


class PractitionerRoleReferenceSerializer(BaseReferenceModelSerializer):
    """Practitioner Role reference serializer."""

    identifier = IdentifierSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = PractitionerRoleReference
        exclude = ["created_at", "updated_at"]
